@@ -22,7 +22,6 @@ try:
 except ImportError:
     argcomplete = None
 
-from copr import CoprClient
 import copr.exceptions as copr_exceptions
 from copr.v3 import (
     Client, config_from_file, CoprException, CoprRequestException,
@@ -74,6 +73,13 @@ Hint: {1}
 
 """
 
+output_format_help = """
+Set the formatting style. We recommend using json, which prints the required data
+in json format. The text format prints the required data in a column, one piece of
+information per line. The text-row format prints all information separated
+by a space on a single line.
+"""
+
 try:
     input = raw_input
 except NameError:
@@ -114,6 +120,10 @@ def buildopts_from_args(args, progress_callback):
         "background": args.background,
         "progress_callback": progress_callback,
     }
+
+    if args.enable_net is not None:
+        buildopts["enable_net"] = ON_OFF_MAP[args.enable_net]
+
     for opt in ["exclude_chroots", "bootstrap", "after_build_id", "with_build_id", "isolation"]:
         value = getattr(args, opt)
         if value is not None:
@@ -121,6 +131,108 @@ def buildopts_from_args(args, progress_callback):
     return buildopts
 
 
+class AbstractPrinter(object):
+    """Abstract class defining mandatory methods of printer classes"""
+
+    def __init__(self, fields):
+        """Represents the data we want to print.
+        Supports callable lambda function which takes exactly one argument"""
+        self.fields = fields
+
+    def add_data(self, data):
+        """Initialize the data to be printed"""
+        raise NotImplementedError
+
+    def finish(self):
+        """Print the data according to the set format"""
+        raise NotImplementedError
+
+
+class RowTextPrinter(AbstractPrinter):
+    """The class takes care of printing the data in row text format"""
+
+    def finish(self):
+        pass
+
+    def add_data(self, data):
+        row_data = []
+        for field in self.fields:
+            if callable(field):
+                row_data.append(str(field(data)))
+            else:
+                row_data.append(str(data[field]))
+        print("\t".join(row_data))
+
+
+class ColumnTextPrinter(AbstractPrinter):
+    """The class takes care of printing the data in column text format"""
+
+    first_line = True
+
+    def finish(self):
+        pass
+
+    def add_data(self, data):
+        if not self.first_line:
+            print()
+        self.first_line = False
+        for field in self.fields:
+            if callable(field):
+                print("{0}: {1}".format(field.__code__.co_varnames[0], str(field(data))))
+            else:
+                print("{0}: {1}".format(field, str(data[field])))
+
+
+class JsonPrinter(AbstractPrinter):
+    """The class takes care of printing the data in json format"""
+
+    def __init__(self, fields):
+        super(JsonPrinter, self).__init__(fields)
+        self.output_info = []
+        self.json_values_list = []
+
+    def __process_fields(self, data):
+        desired_info = []
+        for field in self.fields:
+            if callable(field):
+                desired_info.append(field(data))
+            else:
+                desired_info.append(data[field])
+        self.output_info.append(desired_info)
+
+    def add_data(self, data):
+        json_values = {}
+        self.__process_fields(data)
+        for item in self.output_info:
+            for field, info in zip(self.fields, item):
+                if callable(field):
+                    json_values[field.__code__.co_varnames[0]] = info
+                else:
+                    json_values[field] = info
+        self.json_values_list.append(json_values)
+
+    def finish(self):
+        print(json_dumps(self.json_values_list[0]))
+
+
+class JsonPrinterListCommand(JsonPrinter):
+    """The class takes care of printing the data in list in json format"""
+
+    def finish(self):
+        print(json_dumps(self.json_values_list))
+
+
+def get_printer(output_format, fields, list_command=False):
+    """According to output_format decide which object of printer to return"""
+    if output_format == "json":
+        if list_command:
+            return JsonPrinterListCommand(fields)
+        return JsonPrinter(fields)
+    if output_format == "text":
+        return ColumnTextPrinter(fields)
+    return RowTextPrinter(fields)
+
+
 class Commands(object):
     def __init__(self, config_path):
         self.config_path = config_path or '~/.config/copr'
@@ -265,17 +377,29 @@ class Commands(object):
         if answer == 'y':
             ownername, projectname = self.parse_name(args.name)
 
+            copr_url = self.client.config["copr_url"]
+            api_url = "{0}/api".format(copr_url)
+            url = "{0}/coprs/{1}/{2}/{3}/".format(
+                api_url, ownername, projectname, "new_webhook_secret")
+
+            auth = (self.client.config["login"],
+                    self.client.config["token"])
+
             # @TODO Rewrite this call to the APIv3.
             # @TODO I am not doing it right away because it is a release-blocker
-            # @TODO and I don't have a time to do it right now.
-            client = CoprClient.create_from_file_config()
-            result = client.new_webhook_secret(projectname, ownername=ownername)
+            try:
+                response = requests.post(url=url, auth=auth)
+                result = response.json()
+            except requests.ConnectionError as ex:
+                sys.stderr.write(str(ex) + "\n")
+                return
 
-            if result.output != "ok":
-                sys.stderr.write(result.error + "\n")
+            if result["output"] != "ok":
+                sys.stderr.write(result["error"] + "\n")
                 sys.stderr.write("Un-expected data returned, please report this issue\n")
+                return
 
-            print(result.message)
+            print(result["message"])
 
     @requires_api_auth
     def action_build(self, args):
@@ -442,6 +566,7 @@ class Commands(object):
             delete_after_days=args.delete_after_days,
             multilib=ON_OFF_MAP[args.multilib],
             module_hotfixes=ON_OFF_MAP[args.module_hotfixes],
+            fedora_review=args.fedora_review,
         )
         print("New project was successfully created.")
 
@@ -466,6 +591,7 @@ class Commands(object):
             delete_after_days=args.delete_after_days,
             multilib=ON_OFF_MAP[args.multilib],
             module_hotfixes=ON_OFF_MAP[args.module_hotfixes],
+            fedora_review=ON_OFF_MAP[args.fedora_review],
         )
 
     @requires_api_auth
@@ -515,10 +641,17 @@ class Commands(object):
 
         builds_list = self.client.build_proxy.get_list(ownername, projectname,
                                                        pagination=pagination)
+        if not args.output_format:
+            args.output_format = "text-row"
+            sys.stderr.write(
+                "The default setting will be changed from text-row to json in the following releases\n")
+
         while builds_list:
-            for build in builds_list:
-                print("{0}\t{1}\t{2}".format(build["id"], build["source_package"]["name"],
-                                             build["state"]))
+            fields = ["id", lambda name: name["source_package"]["name"], "state"]
+            printer = get_printer(args.output_format, fields, True)
+            for data in builds_list:
+                printer.add_data(data)
+            printer.finish()
             builds_list = next_page(builds_list)
 
     def action_mock_config(self, args):
@@ -637,7 +770,12 @@ class Commands(object):
         project_chroot = self.client.project_chroot_proxy.get(
             ownername=owner, projectname=copr, chrootname=chroot
         )
-        print(json_dumps(project_chroot))
+        fields = ["additional_packages", "additional_repos", "comps_name", "delete_after_days",
+                  "isolation", "mock_chroot", "ownername", "projectname", "with_opts",
+                  "without_opts"]
+        printer = get_printer(args.output_format, fields)
+        printer.add_data(project_chroot)
+        printer.finish()
 
     def action_list_chroots(self, args):
         """List all currently available chroots.
@@ -753,7 +891,14 @@ class Commands(object):
                                                       with_latest_build=args.with_latest_build,
                                                       with_latest_succeeded_build=args.with_latest_succeeded_build)
         packages_with_builds = [self._package_with_builds(p, args) for p in packages]
-        print(json_dumps(packages_with_builds))
+        fields = ["id", "auto_rebuild", "name", "ownername", "projectname", "source_dict",
+                  "source_type", "latest_succeeded_build", "latest_build"]
+        if args.with_all_builds:
+            fields.insert(1, "builds")
+        printer = get_printer(args.output_format, fields, True)
+        for data in packages_with_builds:
+            printer.add_data(data)
+        printer.finish()
 
     def action_list_package_names(self, args):
         ownername, projectname = self.parse_name(args.copr)
@@ -771,7 +916,13 @@ class Commands(object):
             with_latest_succeeded_build=args.with_latest_succeeded_build,
         )
         package = self._package_with_builds(package, args)
-        print(json_dumps(package))
+        fields = ["auto_rebuild", "id", "latest_build", "latest_succeeded_build", "name", "ownername",
+                  "projectname", "source_dict", "source_type"]
+        if args.with_all_builds:
+            fields.insert(1, "builds")
+        printer = get_printer(args.output_format, fields)
+        printer.add_data(package)
+        printer.finish()
 
     def _package_with_builds(self, package, args):
         ownername, projectname = self.parse_name(args.copr)
@@ -992,6 +1143,13 @@ def setup_parser():
               "able to install 32bit variants of the packages (e.g. i386 for "
               "x86_64 arch), default is 'off'"))
 
+    parser_create.add_argument(
+        "--fedora-review", action="store_true", default=False,
+        help=("When submitting new package to Fedora, it needs to comply with "
+              "Fedora Packaging Guidelines. Use fedora-review tool to help you "
+              "discover packaging errors. Failing fedora-review will not fail "
+              "the build itself."))
+
     parser_create.set_defaults(func="action_create")
 
     # create the parser for the "modify_project" command
@@ -1037,6 +1195,14 @@ def setup_parser():
               "multilib capable architecture (e.g. x86_64), they will also be "
               "able to install 32bit variants of the packages (e.g. i386 for "
               "x86_64 arch), default is \"don't change\""))
+
+    parser_modify.add_argument(
+        "--fedora-review", choices=["on", "off"], default="off",
+        help=("When submitting new package to Fedora, it needs to comply with "
+              "Fedora Packaging Guidelines. Use fedora-review tool to help you "
+              "discover packaging errors. Failing fedora-review will not fail "
+              "the build itself."))
+
     parser_modify.set_defaults(func="action_modify_project")
 
     # create the parser for the "delete" command
@@ -1055,6 +1221,8 @@ def setup_parser():
     parser_builds.add_argument("project", help="Which project's builds should be listed.\
                                Can be just a name of the project or even in format\
                                username/project or @groupname/project.")
+    parser_builds.add_argument("--output-format", choices=["text", "json", "text-row"],
+                               help=output_format_help)
     parser_builds.set_defaults(func="action_list_builds")
 
     #########################################################
@@ -1171,6 +1339,12 @@ def setup_parser():
     parser_build_parent.add_argument("--isolation", choices=["simple", "nspawn", "default"], default="unchanged",
                                      help="Choose the isolation method for running commands in buildroot.")
 
+    parser_build_parent.add_argument(
+        "--enable-net",
+        choices=["on", "off"],
+        help=("If net should be enabled for this build (by default "
+              "the project configuration is used, see 'modify' command)."))
+
     parser_build_parent.add_argument(
         "--bootstrap",
         choices=["unchanged", "default", "on", "off", "image"],
@@ -1314,6 +1488,8 @@ def setup_parser():
 
     parser_get_chroot = subparsers.add_parser("get-chroot", help="Get chroot of a project")
     parser_get_chroot.add_argument("coprchroot", help="Path to a project chroot as owner/project/chroot or project/chroot")
+    parser_get_chroot.add_argument("--output-format", default="json", choices=["text", "json", "text-row"],
+                                   help=output_format_help)
     parser_get_chroot.set_defaults(func="action_get_chroot")
 
     parser_list_chroots = subparsers.add_parser("list-chroots", help="List all currently available chroots.")
@@ -1437,6 +1613,8 @@ def setup_parser():
                                       help="Also display data related to the latest succeeded build for the package.")
     parser_list_packages.add_argument("--with-all-builds", action="store_true",
                                       help="Also display data related to the builds for the package.")
+    parser_list_packages.add_argument("--output-format", default="json", choices=["text", "json", "text-row"],
+                                      help=output_format_help)
     parser_list_packages.set_defaults(func="action_list_packages")
 
     # package names listing
@@ -1460,6 +1638,8 @@ def setup_parser():
                                     help="Also display data related to the latest succeeded build for each package.")
     parser_get_package.add_argument("--with-all-builds", action="store_true",
                                     help="Also display data related to the builds for each package.")
+    parser_get_package.add_argument("--output-format", default="json", choices=["text", "json", "text-row"],
+                                    help=output_format_help)
     parser_get_package.set_defaults(func="action_get_package")
 
     # package deletion
@@ -1492,7 +1672,8 @@ def setup_parser():
 
     # module building
     parser_build_module = subparsers.add_parser("build-module", help="Builds a given module in Copr")
-    parser_build_module.add_argument("copr", help="The copr repo to build module in. Can be just name of project or even in format owner/project.")
+    parser_build_module.add_argument("copr", help="The copr repo to build module in. Can be just name of project "
+                                                  "or even in format owner/project.")
     parser_build_module_mmd_source = parser_build_module.add_mutually_exclusive_group(required=True)
     parser_build_module_mmd_source.add_argument("--url", help="SCM with modulemd file in yaml format")
     parser_build_module_mmd_source.add_argument("--yaml", help="Path to modulemd file in yaml format")
